<?php
/*
 +--------------------------------------------------------------------+
 | CiviCRM version 1.3                                                |
 +--------------------------------------------------------------------+
 | Copyright (c) 2005 Donald A. Lobo                                  |
 +--------------------------------------------------------------------+
 | This file is a part of CiviCRM.                                    |
 |                                                                    |
 | CiviCRM is free software; you can copy, modify, and distribute it  |
 | under the terms of the Affero General Public License Version 1,    |
 | March 2002.                                                        |
 |                                                                    |
 | CiviCRM is distributed in the hope that it will be useful, but     |
 | WITHOUT ANY WARRANTY; without even the implied warranty of         |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
 | See the Affero General Public License for more details.            |
 |                                                                    |
 | You should have received a copy of the Affero General Public       |
 | License along with this program; if not, contact the Social Source |
 | Foundation at info[AT]socialsourcefoundation[DOT]org.  If you have |
 | questions about the Affero General Public License or the licensing |
 | of CiviCRM, see the Social Source Foundation CiviCRM license FAQ   |
 | at http://www.openngo.org/faqs/licensing.html                       |
 +--------------------------------------------------------------------+
*/

/**
 *
 * @package CRM
 * @author Donald A. Lobo <lobo@yahoo.com>
 * @copyright Donald A. Lobo (c) 2005
 * $Id$
 *
 */

define( 'CRM_CORE_BAO_CUSTOMOPTION_VALUE_SEPERATOR',"");

require_once 'CRM/Core/DAO/CustomOption.php';


/**
 * Business objects for managing custom data options.
 *
 */
class CRM_Core_BAO_CustomOption extends CRM_Core_DAO_CustomOption {

       

    /**
     * Takes a bunch of params that are needed to match certain criteria and
     * retrieves the relevant objects. Typically the valid params are only
     * contact_id. We'll tweak this function to be more full featured over a period
     * of time. This is the inverse function of create. It also stores all the retrieved
     * values in the default array
     *
     * @param array $params   (reference ) an assoc array of name/value pairs
     * @param array $defaults (reference ) an assoc array to hold the flattened values
     *
     * @return object CRM_Core_BAO_CustomOption object
     * @access public
     * @static
     */
     function retrieve( &$params, &$defaults )
    {
        $customOption =& new CRM_Core_DAO_CustomOption( );
        $customOption->copyValues( $params );
        if ( $customOption->find( true ) ) {
            CRM_Core_DAO::storeValues( $customOption, $defaults );
            return $customOption;
        }
        return null;
    }

     /**
     * takes an associative array and creates a custom option object
     *
     * This function is invoked from within the web form layer and also from the api layer
     *
     * @param array $params (reference) an assoc array of name/value pairs
     *
     * @return object CRM_Core_DAO_CustomField object
     * @access public
     * @static
     */
     function create(&$params)
    {
        $customOptionBAO =& new CRM_Core_BAO_CustomOption();
        $customOptionBAO->copyValues($params);
        return $customOptionBAO->save();
    }
    
    /**
     * update the is_active flag in the db
     *
     * @param int      $id        id of the database record
     * @param boolean  $is_active value we want to set the is_active field
     *
     * @return Object             DAO object on sucess, null otherwise
     * @static
     */
     function setIsActive( $id, $is_active )
    {
        return CRM_Core_DAO::setFieldValue( 'CRM_Core_DAO_CustomOption', $id, 'is_active', $is_active );
    }


    /**
     * returns all active options ordered by weight for 
     *
     * @param  int      $fieldId         field whose options are needed
     * @param  boolean  $inactiveNeeded  do we need inactive options ?
     *
     * @return array $customOption all active options for fieldId
     * @static
     */
     function getCustomOption($fieldId, $inactiveNeeded=false)
    {       
        $customOptionDAO =& new CRM_Core_DAO_CustomOption();
        $customOptionDAO->entity_id    = $fieldId;
        $customOptionDAO->entity_table = "civicrm_custom_field";
        if (!$inactiveNeeded) {
            $customOptionDAO->is_active = 1;
        }
        $customOptionDAO->orderBy('weight ASC, label ASC');
        $customOptionDAO->find();
        
        $customOption = array();
        while ($customOptionDAO->fetch()) {
            $customOption[$customOptionDAO->id] = array();
            $customOption[$customOptionDAO->id]['id']    = $customOptionDAO->id;
            $customOption[$customOptionDAO->id]['label'] = $customOptionDAO->label;
            $customOption[$customOptionDAO->id]['value'] = $customOptionDAO->value;
        }
        return $customOption;
    }

    /**
     * Function to get the values of the checkboxes
     *
     * param $fieldId integer field id
     *
     * @static
     * @access public
     */
     function getCustomValues($fieldId)
    {
        $customValueDAO =& new CRM_Core_DAO_CustomValue();
        $customValueDAO->custom_field_id = $fieldId;
        $customValueDAO->find(true);
        $values = $customValueDAO->char_data;
        $customValue = array();
        $customValue = explode(CRM_CORE_BAO_CUSTOMOPTION_VALUE_SEPERATOR ,$values);
        return $customValue;
    }

     function getOptionLabel( $fieldId, $value, $fieldType = null ) {
        if ( $fieldType &&
             $fieldType != 'Select' &&
             $fieldType != 'Radio' ) {
            return $value;
        }

        $dao =& new CRM_Core_DAO_CustomOption( );
        $dao->entity_id    = $fieldId;
        $dao->entity_table = 'civicrm_custom_field';
        $dao->value = $value;
        if ( $dao->find( true ) ) {
            return $dao->label;
        }
        return $value;
    }


    /**
     * Function to delete Option
     *
     * param $optionId integer option id
     *
     * @static
     * @access public
     */
     function del($optionId) 
    {
        require_once 'CRM/Core/BAO/CustomField.php';
        require_once 'CRM/Core/BAO/CustomValue.php';
        
        $optionDAO =& new CRM_Core_DAO_CustomOption();
        $optionDAO->id = $optionId;
        $optionDAO->entity_table = "civicrm_custom_field";
        $optionDAO->find();
        $optionDAO->fetch();
        $custom_field_id = $optionDAO->entity_id;
        $value = $optionDAO->value;
        $fieldDAO = & new CRM_Core_DAO_CustomField();
        $fieldDAO->id = $custom_field_id;
        $fieldDAO->find();
        $fieldDAO->fetch();
        
        $customValueDAO = & new CRM_Core_DAO_CustomValue();
        $customValueDeleteDAO = & new CRM_Core_DAO_CustomValue();
        $customValueSaveDAO = & new CRM_Core_DAO_CustomValue();

        //added multiselect in if-statement below
        if( $fieldDAO->html_type !='CheckBox' && $fieldDAO->html_type !='Multi-Select' ) {
            
            $customValueDAO->custom_field_id = $custom_field_id;
            $customValueDAO->find();
            while($customValueDAO->fetch()) {
                if( $customValueDAO->char_data == $value ) {
                    $customValueDeleteDAO->id = $customValueDAO->id;
                    $customValueDeleteDAO->delete();
                }
            }
        } else {
            $customValueDAO->custom_field_id = $custom_field_id;
            $customValueDAO->find();
            while($customValueDAO->fetch()) {
                $optionValues =  $customValueDAO->char_data;
                $customValue = explode(CRM_CORE_BAO_CUSTOMOPTION_VALUE_SEPERATOR ,$optionValues);
                
                if( in_array ($value,$customValue) ) {
                    if( count($customValue) == 1) {
                        $customValueDeleteDAO->id = $customValueDAO->id;
                        $customValueDeleteDAO->delete();
                    } else {
                        $customValueSaveDAO = $customValueDAO;
                        $keyArray = array_keys($customValue,$value);
                        unset($customValue[$keyArray[0]]);
                        $newCustomValue = implode(CRM_CORE_BAO_CUSTOMOPTION_VALUE_SEPERATOR,$customValue);
                        $customValueSaveDAO->char_data = $newCustomValue;
                        $customValueSaveDAO->save();
                    }
                }
            }
            
        } 
        
        $optionDAO->delete();
        return null;
    }

    /**
     * return the custom options associated with a specific entity id/table
     * as a name/value pair
     *
     * @param string $entity_table name of the table
     * @param string $entity_id   
     * @param array  $values       array tos tore the options in
     *
     * @return void
     * @static
     */
     function getAssoc( $entity_table, $entity_id, &$values ) {
        require_once 'CRM/Core/DAO/CustomOption.php';  
        $dao =& new CRM_Core_DAO_CustomOption( );  
        $dao->entity_table = $entity_table;
        $dao->entity_id    = $entity_id;  
        $dao->find( ); 

        // now extract the amount 
        $values['value'] = array( ); 
        $values['label'] = array( ); 
        $index  = 1; 
         
        while ( $dao->fetch( ) ) { 
            $values['value'][$index] = $dao->value; 
            $values['label'][$index] = $dao->label; 
            $index++; 
        } 
    }

}
?>
