<?php
// $Id: update.php,v 1.147 2005/04/06 19:04:02 dries Exp $

/**
 * @file
 * Administrative page for handling updates from one Drupal version to another.
 *
 * Point your browser to "http://www.site.com/update.php" and follow the
 * instructions.
 *
 * If you are not logged in as administrator, you will need to modify the access
 * check statement below. Change the TRUE into a FALSE to disable the access
 * check. After finishing the upgrade, be sure to open this file and change the
 * FALSE back into a TRUE!
 */

// Disable access checking?
$access_check = TRUE;

$default_update = "";

if (!ini_get("safe_mode")) {
  set_time_limit(360);
}

include_once "database/updates.inc";

function update_data($start) {
  global $sql_updates;
  $sql_updates = array_slice($sql_updates, ($start-- ? $start : 0));
  foreach ($sql_updates as $date => $func) {
    print "<strong>$date</strong><br />\n<pre>\n";
    $ret = $func();
    foreach ($ret as $return) {
      print $return[1];
      print $return[2];
    }
    variable_set("update_start", $date);
    print "</pre>\n";
  }
  db_query('DELETE FROM {cache}');
}

function update_page_header($title) {
  $output = "<html><head><title>$title</title>";
  $output .= <<<EOF
      <link rel="stylesheet" type="text/css" media="print" href="misc/print.css" />
      <style type="text/css" title="layout" media="Screen">
        @import url("misc/drupal.css");
      </style>
EOF;
  $output .= "</head><body>";
  $output .= "<div id=\"logo\"><a href=\"http://drupal.org/\"><img src=\"misc/druplicon-small.png\" alt=\"Druplicon - Drupal logo\" title=\"Druplicon - Drupal logo\" /></a></div>";
  $output .= "<div id=\"update\"><h1>$title</h1>";
  return $output;
}

function update_page_footer() {
  return "</div></body></html>";
}

function update_page() {
  global $user, $sql_updates;

  if (isset($_POST['edit'])) {
    $edit = $_POST['edit'];
  }
  if (isset($_POST['op'])) {
    $op = $_POST['op'];
  }

  switch ($op) {
    case "Update":
      // make sure we have updates to run.
      print update_page_header("Drupal database update");
      $links[] = "<a href=\"index.php\">main page</a>";
      $links[] = "<a href=\"index.php?q=admin\">administration pages</a>";
      print theme("item_list", $links);
      print "<br />Updates were attempted. If you see no failures below, you may proceed happily to the <a href=\"index.php?q=admin\">administration pages</a>.";
      print " Otherwise, you may need to update your database manually.<br/><br/>\n";
      print 'IMPORTANT: If, for the update, you had to set write permissions for the directory <em><code>sites</code></em> and its subdirectories, you will now need to take away those permissions. <a href="http://civicspacelabs.org/update_reqs/082/sites_perms_off" target="_blank">More help</a>.<br/><br/><br/>';
        // NOTE: we can't use l() here because the URL would point to 'update.php?q=admin'.
      if ($edit["start"] == -1) {
        print "No updates to perform.";
      }
      else {
        update_data($edit["start"]);
      }
      print update_page_footer();
      break;
    case "Try Again":
    case "Run Checks":
      $failed_conditions = update_check_conditions($edit["start"]);
      if (count($failed_conditions)) {
        print update_page_header('Update: Some checks failed.  Please resolve before continuing.');
        print "<p>One or more of the updates you are running require you to ensure certain conditons are met.  Please read through the list below and make the necessary changes.  Once you've fixed these problems, click the 'Try Again' button below.";
        print "<ul>\n";
        foreach ($failed_conditions as $failure) {
          print "<li>";
          print $failure['msg'];
          print "  ";
          print '<a href="'. $failure['link'] .'" target="_blank">More help</a>';
          print "</li>\n";
        }
        print "</ul>\n";
        print form(form_hidden('start', $edit["start"]) . form_submit('Try Again'));
        print update_page_footer();
      }
      else {
        print update_page_header('Update: Checks complete.  Proceed to database update.');
        print "<p>You have satisfied all the checks and may continute to the upgrade process.  Before doing anything, however, you should backup your database. This process will change your database and its values, and some things might get lost.</p>\n";
        print "Don't upgrade your database twice as it may cause problems.</p></li>\n";
        print form(form_hidden('start', $edit["start"]) . form_submit('Update'));
        print update_page_footer();
      }
      break;
    default:
    // We do checks after selecting the update path on TRUNK.
/*      $failed_conditions = update_check_conditions();
      if (count($failed_conditions)) {
        // Use this place to say that conditions must be fulfilled before they can actually continue
        print update_page_header('Update: Some checks failed.  Please resolve before continuing.');
        print "<p>One or more of the updates you are running require you to ensure certain conditons are met.  Please read through the list below and make the necessary changes.  Once you've fixed these problems, click the 'Try Again' button below.";
        print "<ul>\n";
        foreach ($failed_conditions as $failure) {
          print "<li>";
          print $failure['msg'];
          print "  ";
          print '<a href="'. $failure['link'] .'" target="_blank">More help</a>';
          print "</li>\n";
        }
        print "</ul>\n";
        print form(form_submit('Try Again'));
        print update_page_footer();
      }
      else {*/
        
        $dates = create_updates();
        // make update form and output it.
        $form = form_select("Perform updates from", "start", (isset($default_update) ? $default_update : -1), $dates, "This defaults to the first available update since the last update you performed.");
        $form .= form_submit("Run Checks");
        print update_page_header("Drupal database update");
        print form($form);
        print update_page_footer();
      //} broken if statement from line 116.
      break;
  }
}

function create_updates() {
  global $sql_updates;
  // NOTE: We need the following five lines in order to fix a bug with
  //       database.mysql (issue #15337).  We should be able to remove
  //       this work around in the future.
  $result = db_query("SELECT * FROM {variable} WHERE name = 'update_start' AND value LIKE '%;\"'");
  if ($variable = db_fetch_object($result)) {
    $variable->value = unserialize(substr($variable->value, 0, -2) .'";');
    variable_set('update_start', $variable->value);
  }

  $start = variable_get("update_start", 0);
  //$dates[] = "All";
  $i = 1;
  foreach ($sql_updates as $date => $sql) {
    $dates[$i++] = $date;
    if ($date == $start) {
      $default_update = $i;
    }
  }
  $dates[$i] = "No updates available";
  return $dates;
}

function update_info() {
  print update_page_header("Drupal database update");
  print "<ol>\n";
  print "<li>Use this script to <strong>upgrade an existing Drupal installation</strong>.  You don't need this script when installing Drupal from scratch.</li>";
  print "<li>Before doing anything, backup your database. This process will change your database and its values, and some things might get lost.</li>\n";
  print "<li>Update your Drupal sources, check the notes below and <a href=\"update.php?op=update\">run the database upgrade script</a>.  Don't upgrade your database twice as it may cause problems.</p></li>\n";
  print "<li>Go through the various administration pages to change the existing and new settings to your liking.</li>\n";
  print "</ol>";
  print "<h3>Notes:</h3>";
  
  // ---
  // Tell user conditions for upgrading that are involved in adding CiviCRM in CS 0.8.2
  print "<p>If you are <strong>upgrading to or through CivicSpace 0.8.2</strong> you will need to satisfy the following conditions before being able to upgrade:</p>";
  print "<ul>\n";
  print "<li>You will need to make sure you are using <em><code>MySQL 4.0 or later</code></em>.</li>\n";
  print "<li>You will need to make sure that the file <em><code>". update_conf_init() . '/civicrm.settings.php' ."</code></em> is writable.</li>\n";
  print "<li>You will need to make sure that the directory <em><code>files/</code></em> is readable and writable.</li>\n";
  print "<li>You will need to make sure that the directory <em><code>files/civicrm</code></em> is readable and writable.</li>\n";
  print "<li>You will need to make sure that the directory <em><code>files/civicrm/upload</code></em> is readable and writable.</li>\n";
  print "<li>You will need to make sure that the directory <em><code>files/civicrm/templates_c</code></em> is readable and writable.</li>\n";
  print "</ul>\n";
  // ---
  
  print " <p>If you <strong>upgrade from CivicSpace 0.7.x</strong>, you will need to create the <code>users_roles</code> and <code>locales_meta</code> tables manually before upgrading. To create these tables, issue the following SQL commands:</p>

  <pre>
  CREATE TABLE users_roles (
    uid int(10) unsigned NOT NULL default '0',
    rid int(10) unsigned NOT NULL default '0',
    PRIMARY KEY (uid, rid)
  );
  CREATE TABLE locales_meta (
    locale varchar(12) NOT NULL default '',
    name varchar(64) NOT NULL default '',
    enabled int(2) NOT NULL default '0',
    isdefault int(2) NOT NULL default '0',
    plurals int(1) NOT NULL default '0',
    formula varchar(128) NOT NULL default '',
    PRIMARY KEY  (locale)
  );
  </pre>
  </li>";
  print update_page_footer();
}

/**
 * The folowing function is a straight copy of the function conf_init() defined in bootstrap.inc
 *
 */

function update_conf_init() {
  static $conf = '';

  if ($conf) {
    return $conf;
  }

  $confdir = 'sites';
  $uri = explode('/', $_SERVER['PHP_SELF']);
  $server = explode('.', rtrim($_SERVER['HTTP_HOST'], '.'));
  for ($i = count($uri) - 1; $i > 0; $i--) {
    for ($j = count($server); $j > 0; $j--) {
      $dir = implode('.', array_slice($server, -$j)) . implode('.', array_slice($uri, 0, $i));
      if (file_exists("$confdir/$dir/settings.php")) {
        $conf = "$confdir/$dir";
        return $conf;
      }
    }
  }
  $conf = "$confdir/default";
  return $conf;
}

function update_check_conditions($start) {
  global $sql_updates;
  $dates = create_updates();
  $thedate = $dates[$start];

  $failures = array();
  // CONDITIONS FOR civic_update_14(): CS 0.8.1.3 --> CS 0.8.2
  
  $sql_updates = array_slice($sql_updates, ($start-- ? $start : 0));
  foreach ($sql_updates as $date => $func) { 
    switch ($func) {
    case "civic_update_14": // CS 0.8.1.3 --> CS 0.8.2
      // 1st condition for civic_update_14(): Make sure we have MySQL 4.0 or higher.
      $db_version_check = mysql_fetch_object(mysql_query('SELECT VERSION() AS version'));
      if ($last = strrpos($db_version_check->version, '_')) {
        $db_version_check->version = substr($db_version_check->version, 0, $last);
      }
      $mysql_version = explode('.', $db_version_check->version);
      if ($mysql_version[0] < 4) {
        $failures[] = array('msg' => 'The version of MySQL for your site\'s database is older than 4.0.  You must upgrade your MySQL to a version of <em><code>MySQL 4 or newer</code></em>.', 'link' => 'http://civicspacelabs.org/update_reqs/082/mysql_version');
      }      
      // 2nd condition for civic_update_14(): Make sure civicrm.php file in sites/default/ is readable and writable.
      $civicrm_filename = conf_init(). '/civicrm.php';
      if (file_exists($civicrm_filename)) {
        if (!is_writable($civicrm_filename)) {
          $failures[] = array('msg' => 'The file <em><code>'. $civicrm_filename .'</code></em> is not writable.  Before you can proceed with the update, you need to change the permissions for this file so that it is writable.  You will want to make sure you take away write permissions for this file when you are done. (Don\'t worry; we\'ll remind you.)', 'link' => 'http://civicspacelabs.org/update_reqs/082/civicrmconf_perms');
        }
      }      
      // 3rd condition for civic_update_14()
      if (!is_writable('files')) {
        $failures[] = array('msg' => 'The directory <em><code>files/</code></em> is not writable.  Before you proceed with the update, you need to allow this directory to be readable and writable.', 'link' => 'http://civicspacelabs.org/update_reqs/082/files_perms');
      }     
      // 4th condition for civic_update_14()  
      if (!is_writable('files/civicrm')) {
        $failures[] = array('msg' => 'The directory <em><code>files/civicrm</code></em> is not writable.  You must set the permissions of this directory (and create it first if it doesn\' exist.', 'link' => 'http://civicspacelabs.org/update_reqs/082/file_civicrm_perms');
      }        
      // 5th condition for civic_update_14()
      if (!is_writable('files/civicrm/upload')) {
        $failures[] = array('msg' => 'The directory <em><code>files/civicrm/upload</code></em> is not writable.  You need to allow this directory to be readable and writable before you proceed.  Please create it first if it does not yet exist.', 'link' => 'http://civicspacelabs.org/update_req/082/file_civicrm_upload_perms');
      }  
      // 6th condition for civic_update_14()
      if (!is_writable('files/civicrm/templates_c')) {
        $failures[] = array('msg' => 'The directory <em><code>files/civicrm/templates_c</code></em> is not writable.  You need to allow this directory toi be readable and writable before you proceed.  Please create it first if it does not yet exist.', 'link' => 'http://civicspacelabs.org/update_req/082/file_civicrm_templates_c_perms');
      }  
    
    
    case "civic_update_15": //CS 0.8.2 --> CS 0.8.3
      if (!is_writable(conf_init())) {
        $failures[] = array('msg' => 'The directory <em><code>'. conf_init() .'</code></em> is not writable.  Before you proceed with the update, you need to allow this directory to be readable and writable.', 'link' => 'about:blank');
      }     
    
    }    
  } 
  
  return $failures;
}

if (isset($_GET["op"])) {
  include_once "includes/bootstrap.inc";
  include_once "includes/common.inc";

  // Access check:
  if (($access_check == 0) || ($user->uid == 1)) {
    update_page();
  }
  else {
    print update_page_header("Access denied");
    print "<p>Access denied.  You are not authorized to access this page.  Please log in as the admin user (the first user you created). If you cannot log in, you will have to edit <code>update.php</code> to bypass this access check.  To do this:</p>";
    print "<ol>";
    print " <li>With a text editor find the update.php file on your system. It should be in the main Drupal directory that you installed all the files into.</li>";
    print " <li>There is a line near top of update.php that says <code>\$access_check = TRUE;</code>. Change it to <code>\$access_check = FALSE;</code>.</li>";
    print " <li>As soon as the script is done, you must change the update.php script back to its original form to <code>\$access_check = TRUE;</code>.</li>";
    print " <li>To avoid having this problem in future, remember to log in to your website as the admin user (the user you first created) before you backup your database at the beginning of the update process.</li>";
    print "</ol>";

    print update_page_footer();
  }
}
else {
  update_info();
}
?>
